/*
 * Module stderror.c
 *
 * (C) by Andreas Theofilu (Theos Soft)
 * This source is protected by the GNU General Public License (GPL)!
 */
#include <stdio.h>
#ifdef Linux
   #include <errno.h>
   #include <stdlib.h>
   #include <string.h>
#endif
#include <varargs.h>
#include "systype.h"

#ifdef CLIENT
#ifdef SERVER
cant define both CLIENT and SERVER
#endif
#endif

#ifndef CLIENT
#ifndef SERVER
#define CLIENT 1
#endif
#endif

#ifndef NULL
#define NULL	((void *) 0)
#endif

#include "stderror.h"

char *pname = NULL;

#ifdef CLIENT			/* these all output to stderr */

void err_quit (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);

	if (pname != NULL)
	   fprintf (stderr, "%s: ", pname);

	fmt = va_arg (args, char *);
	vfprintf (stderr, fmt, args);
	fputc ('\n', stderr);
	va_end (args);
	exit (1);
}

void err_sys (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);

	if (pname != NULL)
	   fprintf (stderr, "%s: ", pname);

	fmt = va_arg (args, char *);
	vfprintf (stderr, fmt, args);
	va_end (args);
	my_perror ();
	exit (1);
}

void err_ret (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);

	if (pname != NULL)
	   fprintf (stderr, "%s: ", pname);

	fmt = va_arg (args, char *);
	vfprintf (stderr, fmt, args);
	va_end (args);
	my_perror ();
	fflush (stdout);
	fflush (stderr);
	return;
}

void err_dump (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);

	if (pname != NULL)
	   fprintf (stderr, "%s: ", pname);

	fmt = va_arg (args, char *);
	vfprintf (stderr, fmt, args);
	va_end (args);
	my_perror ();
	fflush (stdout);
	fflush (stderr);
	abort ();		/* dump core and terminate */
	exit (1);		/* shouldn't get here */
}

void my_perror ()
{
char *sys_err_str ();

	fprintf (stderr, " %s\n", sys_err_str ());
}

#endif		/* CLIENT */

#ifdef SERVER

#if defined (BSD) || defined (sinix) || defined (Linux)
#include <syslog.h>
#else
#define syslog(a,b)		fprintf(stderr, "%s\n", (b))
#define openlog(a,b,c)		fprintf(stderr, "%s\n", (a))
#endif	/* BSD */

char emesgstr[255] = {0};	/* used by all server routines */

void err_init (char *ident)
{
	openlog (ident, (LOG_PID | LOG_CONS), LOG_DAEMON);
}

void err_quit (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);
	fmt = va_arg (args, char *);
	vsprintf (emesgstr, fmt, args);
	va_end (args);

	syslog (LOG_ERR, emesgstr);

	exit (1);
}

void err_sys (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);
	fmt = va_arg (args, char *);
	vsprintf (emesgstr, fmt, args);
	va_end (args);

	my_perror ();
	syslog (LOG_ERR, emesgstr);

	exit (1);
}

void err_ret (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);
	fmt = va_arg (args, char *);
	vsprintf (emesgstr, fmt, args);
	va_end (args);

	my_perror ();
	syslog (LOG_ERR, emesgstr);

	return;
}

void err_dump (va_alist)
va_dcl
{
va_list	args;
char	*fmt;

	va_start (args);
	fmt = va_arg (args, char *);
	vsprintf (emesgstr, fmt, args);
	va_end (args);

	my_perror ();
	syslog (LOG_ERR, emesgstr);

	abort ();		/* dump core and terminate */
	exit (1);		/* shouldn't get here */
}

void my_perror ()
{
register int len;
char *sys_err_str ();

	len = strlen (emesgstr);
	sprintf (emesgstr + len, " %s", sys_err_str ());
}

#endif /* SERVER */

extern int	errno;
extern int	sys_nerr;

#if !defined(Linux)
extern char	*sys_errlist[];
#endif

#if defined(SYS5) && !defined(Linux)
int	t_errno;

int	t_nerr;
extern char	*t_errlist;
#endif	/* SYS5 */

/*
 * Return a string containing some additional operating-system
 * dependent information.
 * Note that different versions of UNIX assign different meanings
 * to the same value of "errno" (compare errno's starting with 35
 * between System V and BSD, for example). This means that if an error
 * condition is being sent to another UNIX system, we must interpret
 * the errno value on the system that generated the error, and not
 * just send the decimal value of errno to the other system.
 */

char *sys_err_str ()
{
static char msgstr[200];

	if (errno != 0)
	{
	   if (errno > 0 && errno < sys_nerr)
	      sprintf (msgstr, "(%s)", sys_errlist[errno]);
	   else
	      sprintf (msgstr, "(errno = %d)", errno);
	}
	else
	{
	   msgstr[0] = '\0';
	}
#if defined(SYS5) && !defined(Linux)
	if (t_errno != 0)
	{
	   char tmsgstr[100];

	   if (t_errno > 0 && t_errno < sys_nerr)
	      sprintf (tmsgstr, " (%s)", t_errlist[t_errno]);
	   else
	      sprintf (tmsgstr, ", (t_errno = %d)", t_errno);

	   strcat (msgstr, tmsgstr);
	}
#endif	/* SYS5 */

	return msgstr;
}

int	h_errno;
int	h_nerr;
char	*h_errlist[1];

char *host_err_str ()
{
static char msgstr[200];

	if (h_errno != 0)
	{
	   if (h_errno > 0 && h_errno < h_nerr)
	      sprintf (msgstr, "(%s)", h_errlist[h_errno]);
	   else
	      sprintf (msgstr, "(h_errno = %d)", h_errno);
	}
	else
	{
	   msgstr[0] = '\0';
	}

	return msgstr;
}
